/*
	Licensed Material - Property of IBM

	(C) Copyright 2015 IBM Corp.

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/


'use strict';

var fs = require('fs');
var strings = require('ibm-strings');
var log = require('npmlog');
var path = require('path');
var async = require('async');
var AdmZip = require('adm-zip');
var Q = require('q');
var consts = require('./constants');
var _ = require('lodash');
var exec = require('mdo-exec').run;
var externalizedStrings = require('./externalizedStrings');

log.level = global.MDO_CLI_LOG_LEVEL ? global.MDO_CLI_LOG_LEVEL : log.level;

module.exports.run = function(appDetails, configDetails) {
  var deferred = Q.defer();

  var projectFolder = path.join(process.cwd(), appDetails.appName);

  try {
    createIosFolder(projectFolder);
    async.series([
      function(callback) {
        // Modify the project to change the app related details
        async.parallel([
          modifyProjectName.bind(null, appDetails),
          modifyInfoPlist.bind(null, appDetails),
          modifyMfpClient.bind(null, appDetails, configDetails),
        ], function(err) {
            if (err) {
              callback(err);
            } else {
              callback();
            }
        })
      },
      function(callback) {
        // Parse the push template json to get the push configuration details
        if (appDetails.services.includes(consts.PUSH)) {
          const pushFile = fs.readFileSync(path.join(__dirname, '..', 'templates', 'push.json'));
          const pushJson = JSON.parse(pushFile);        
          async.parallel([
            modifyPodFile.bind(null, pushJson.ios.pod, appDetails),
            modifyPushViewController.bind(null, appDetails),              
            modifyPushAppDelegate.bind(null, appDetails),
            copyAPNSCertificate.bind(null, appDetails, configDetails),
          ], function(err) {
              if (err) {
                callback(err);
              } else {
                callback();
              }
          })
        } else {
          callback();
        }
      },
      function(callback) {
        // Parse the live update template json get the live update configuration details
        if (appDetails.services.includes(consts.LIVEUPDATE)) {
          const luFile = fs.readFileSync(path.join(__dirname, '..', 'templates', 'liveupdate.json'));
          const luJson = JSON.parse(luFile);        
          async.parallel([
            modifyPodFile.bind(null, luJson.ios.pod, appDetails),
            modifyLUViewController.bind(null, appDetails, configDetails),            
          ], function(err) {
              if (err) {
                callback(err);
              } else {
                callback();
              }
          })
        } else {
          callback();
        }
      },
      function(callback) {
        if (appDetails.services.includes(consts.ANALYTICS)) {      
          async.parallel([
            modifyAnalyticsViewController.bind(null, appDetails),            
          ], function(err) {
              if (err) {
                callback(err);
              } else {
                callback();
              }
          })
        } else {
          callback();
        }
      }             
    ], function(err) {
      if (err) {
        strings.error(err.message);
        deferred.reject(err);
      } else {
        // Do a pod install on mac system
        podInstall(appDetails).then(function () {
          // Create the app registration payload for writing in app_config.json
          var payload = {
            applicationKey: {
              version: appDetails.appVersion,
              clientPlatform: 'ios',
              bundleId: appDetails.appId,              
            },
            displayName: appDetails.appName,
          };          
          deferred.resolve(payload);
        }).catch(function (error) {
          deferred.reject(error);
        });
      }
    });
  } catch (error) {
    deferred.reject(error);
  }

  return deferred.promise;
};

// Extract the ios template zip to create ios project folder
function createIosFolder(projectFolder) {
  try {    
    var iosZipPath = path.join(__dirname, '..', 'templates', 'ios.zip');
    var iosZip = new AdmZip(iosZipPath);
    iosZip.extractAllTo(projectFolder, true, true);
  } catch (error) {
    strings.error(error.message);
    throw error;
  }
}

// Modify the app name in all the required files
function modifyProjectName(appDetails, callback) {
  var projectFolder = path.join(process.cwd(), appDetails.appName);

  var renamePaths = [
    path.join(projectFolder, 'MFPStarterIOSSwift'),
    path.join(projectFolder, 'MFPStarterIOSSwift.xcodeproj'),
  ]

  var replacePaths = [
    path.join(projectFolder, `${appDetails.appName}.xcodeproj`, 'project.pbxproj'),
    path.join(projectFolder, appDetails.appName, 'Base.lproj', 'Main.storyboard'),
    path.join(projectFolder, 'Podfile'),
  ]

  renamePaths.forEach(function(renamePath) {
    const destPath = renamePath.replace('MFPStarterIOSSwift', appDetails.appName);
    fs.renameSync(renamePath, destPath); 
  })  
  replacePaths.forEach(function(replacePath) {
    let data = fs.readFileSync(replacePath);
    data = data.toString().replace(/MFPStarterIOSSwift/g, appDetails.appName);
    data = data.toString().replace(/com.ibm.mfpstarteriosswift/g, appDetails.appId);
    fs.writeFileSync(replacePath, data);
  })
  callback();
}

// Modify the info.plist to change the version
function modifyInfoPlist(appDetails, callback) {
  var projectFolder = path.join(process.cwd(), appDetails.appName);
  var replacePath = path.join(projectFolder, appDetails.appName, 'Info.plist');
  let data = fs.readFileSync(replacePath);
  data = data.toString().replace(/1.0/g, appDetails.appVersion);
  fs.writeFileSync(replacePath, data);

  callback();
}

// Modify the mfpclient.plist to change the server details
function modifyMfpClient(appDetails, configDetails, callback) {
  if (configDetails && configDetails.serverConfig) {
    var projectFolder = path.join(process.cwd(), appDetails.appName);
    const mfpClientPath = path.join(projectFolder, 'mfpclient.plist');    

    var server = JSON.parse(fs.readFileSync(configDetails.serverConfig));
    if (server.mfp) {    
      let data = fs.readFileSync(mfpClientPath);
      data = data.toString().replace(/http/g, server.mfp.protocol);
      data = data.toString().replace(/127.0.0.1/g, server.mfp.host);
      data = data.toString().replace(/9080/g, server.mfp.port);
      fs.writeFileSync(mfpClientPath, data);
    }
  }

  callback();
}

// Do a pod install on mac
function podInstall(appDetails) {
  var deferred = Q.defer();

  var projectFolder = path.join(process.cwd(), appDetails.appName);

  exec('pod', ['--version'], projectFolder).then(function() {
    strings.info(externalizedStrings.podInstalling);
    exec('pod', ['install'], projectFolder).then(function() {
      deferred.resolve();
    }, function(err) {
      strings.warn(strings.format(externalizedStrings.podInstallFailed, JSON.stringify(err)));
      deferred.resolve();
    });    
  }, function(err) {
    strings.warn(externalizedStrings.podNotInstalled);
    deferred.resolve();
  });

  return deferred.promise;
}

// Modify the pod file to include service dependencies
function modifyPodFile(iosPod, appDetails, callback) {
  var projectFolder = path.join(process.cwd(), appDetails.appName);
  var podPath = path.join(projectFolder, 'Podfile'); 
  log.verbose(consts.LOG_PREFIX, 'Reading file: ' + podPath);
  fs.readFile(podPath, 'utf-8', function (podErr, podData)  {
    if (podErr) {
      callback(podErr);
    } else {
      let tempIndex = podData.indexOf("post_install");
      let tempStr = podData.substring(0, tempIndex - 1);   
      let replaceStr = tempStr;
      iosPod.forEach((podStmt) => {
        replaceStr = replaceStr.concat(` ${podStmt}\n `);
      });
      podData = podData.replace(tempStr, replaceStr);    
      fs.writeFile(podPath, podData, (err) => {
        if (err) {
          callback(err);
        } else {
          log.verbose(consts.LOG_PREFIX, 'Written file: ' + podPath);
          callback();
        }
      });            
    }
  });
}

// Copy the apns certificate to the mfpconfig folder
function copyAPNSCertificate(appDetails, configDetails, callback) {
  if (configDetails && configDetails.appConfig) {    
    var projectFolder = path.join(process.cwd(), appDetails.appName);  
    var appConfig = JSON.parse(fs.readFileSync(configDetails.appConfig));
    if (appConfig.services) {  
      for (var index = 0; index < appConfig.services.length; index++) {
        var service = appConfig.services[index];
        if (service.name = 'push' && service.settings) {
          if (service.settings.apns && service.settings.apns.certificate) {
            log.verbose(consts.LOG_PREFIX, 'Copying file: ' + service.settings.apns.certificate);
            var configFolder = path.join(projectFolder, consts.MFPCONFIG, path.basename(service.settings.apns.certificate));
            fs.copyFileSync(service.settings.apns.certificate, configFolder);
          }
          break;
        }
      }
    }
  }
  callback();
}

// Modify the ViewController file to include push code snippets

// TODO - a) Revisit below methods to make the parsing robust instead of string manipulation
//        b) Get the code snippet from json template
function modifyPushViewController(appDetails, callback) {
  var projectFolder = path.join(process.cwd(), appDetails.appName);
  var viewPath = path.join(projectFolder, appDetails.appName, 'ViewController.swift'); 
  log.verbose(consts.LOG_PREFIX, 'Reading file: ' + viewPath);
  fs.readFile(viewPath, 'utf-8', function (viewErr, viewData)  {
    if (viewErr) {
      callback(viewErr);
    } else {
      let tempIndex = viewData.indexOf("class");
      let tempStr = viewData.substring(0, tempIndex - 1);   
      let replaceStr = tempStr.concat(consts.PUSH_IOS_IMPORTS);
      viewData = viewData.replace(tempStr, replaceStr);    

      tempIndex = viewData.indexOf("}");
      tempStr = viewData.substring(0, tempIndex - 1);     
      replaceStr = tempStr.concat(consts.PUSH_IOS_IMPL_CALL);    
      viewData = viewData.replace(tempStr, replaceStr);

      tempIndex = viewData.indexOf("}");
      tempStr = viewData.substring(0, tempIndex + 1);     
      replaceStr = tempStr.concat(consts.PUSH_IOS_IMPL);
  
      viewData = viewData.replace(tempStr, replaceStr); 

      tempStr = viewData.substring(0, viewData.lastIndexOf("}"));
      replaceStr = tempStr.concat(consts.PUSH_IOS_HANDLER);   
      viewData = viewData.replace(tempStr, replaceStr);   

      fs.writeFile(viewPath, viewData, (err) => {
        if (err) {
          callback(err);
        } else {
          log.verbose(consts.LOG_PREFIX, 'Written file: ' + viewPath);
          callback();
        }
      });            
    }
  });
}

// Modify the AppDelegate file to include push code snippets
function modifyPushAppDelegate(appDetails, callback) {
  var projectFolder = path.join(process.cwd(), appDetails.appName);
  var viewPath = path.join(projectFolder, appDetails.appName, 'AppDelegate.swift'); 
  log.verbose(consts.LOG_PREFIX, 'Reading file: ' + viewPath);
  fs.readFile(viewPath, 'utf-8', function (viewErr, viewData)  {
    if (viewErr) {
      callback(viewErr);
    } else {
      let tempIndex = viewData.indexOf("@UIApplicationMain");
      let tempStr = viewData.substring(0, tempIndex - 1);   
      let replaceStr = tempStr.concat(consts.PUSH_IOS_IMPORTS);
      viewData = viewData.replace(tempStr, replaceStr);        

      tempIndex = viewData.indexOf("}");
      tempStr = viewData.substring(0, tempIndex + 1);     
      replaceStr = tempStr.concat(consts.PUSH_IOS_DELEGATE_METHODS);                                 
 
      viewData = viewData.replace(tempStr, replaceStr); 

      tempIndex = viewData.indexOf("return");
      tempStr = viewData.substring(0, tempIndex - 1);     
      replaceStr = tempStr.concat(`\n        if let userInfo = launchOptions?[UIApplication.LaunchOptionsKey.remoteNotification] as? [AnyHashable: Any] {
            showNotification(userInfo: userInfo)       
        }\n       `);                   
      viewData = viewData.replace(tempStr, replaceStr);        

      fs.writeFile(viewPath, viewData, (err) => {
        if (err) {
          callback(err);
        } else {
          log.verbose(consts.LOG_PREFIX, 'Written file: ' + viewPath);
          callback();
        }
      });            
    }
  });
}

// Modify the ViewController file to include live update code snippets
function modifyLUViewController(appDetails, configDetails, callback) {
  var projectFolder = path.join(process.cwd(), appDetails.appName);
  var viewPath = path.join(projectFolder, appDetails.appName, 'ViewController.swift'); 
  log.verbose(consts.LOG_PREFIX, 'Reading file: ' + viewPath);
  fs.readFile(viewPath, 'utf-8', function (viewErr, viewData)  {
    if (viewErr) {
      callback(viewErr);
    } else {
      let tempIndex = viewData.indexOf("class");
      let tempStr = viewData.substring(0, tempIndex - 1);   
      let replaceStr = tempStr.concat(consts.LIVEUPDATE_IOS_IMPORTS);
      viewData = viewData.replace(tempStr, replaceStr);    

      tempIndex = viewData.indexOf("}");
      tempStr = viewData.substring(0, tempIndex - 1);     
      replaceStr = tempStr.concat(consts.LIVEUPDATE_IOS_IMPL_CALL);    
      viewData = viewData.replace(tempStr, replaceStr);

      tempIndex = viewData.indexOf("}");
      tempStr = viewData.substring(0, tempIndex + 1);     
      replaceStr = tempStr.concat(consts.LIVEUPDATE_IOS_IMPL);
      viewData = viewData.replace(tempStr, replaceStr); 

      let liveUpdateSchema = '';
      if (configDetails && configDetails.appConfig) {
        const appConfig = JSON.parse(fs.readFileSync(configDetails.appConfig));
        if (appConfig.services) {
          appConfig.services.forEach(function(service) {
            if (service.name == consts.LIVEUPDATE && service.schema) {
              liveUpdateSchema = liveUpdateSchema.concat(`var luArray: [String : Any] = [String : Any]()\n`);
            
              if (service.schema.features) {
                Object.keys(service.schema.features).forEach(function(schemaFeature) {
                  const feature = service.schema.features[schemaFeature];
                  liveUpdateSchema = liveUpdateSchema.concat(`          if let ${feature.name} = configuration?.isFeatureEnabled("${feature.name}") {
              luArray["${feature.name}"] = ${feature.name}\n          }\n`);
                })
              }
              if (service.schema.properties) {
                Object.keys(service.schema.properties).forEach(function(schemaProperty) {
                  const property = service.schema.properties[schemaProperty];
                  liveUpdateSchema = liveUpdateSchema.concat(`          if let ${property.name} = configuration?.getProperty("${property.name}") {
              luArray["${property.name}"] = ${property.name}\n          }\n`);             
                })
              } 
              liveUpdateSchema = liveUpdateSchema.concat(`          let data = try? JSONSerialization.data(withJSONObject: luArray, options: [.prettyPrinted])\n`);           
              liveUpdateSchema = liveUpdateSchema.concat(`          changeStatusText("Live update schema:" + String(data: data!,  encoding: .utf8)!)`);           
            }
          });
        }
      }

      tempStr = viewData.substring(0, viewData.lastIndexOf("}"));
      replaceStr = tempStr.concat(consts.LIVEUPDATE_IOS_HANDLER);    
      replaceStr = replaceStr.replace('{liveUpdateSchema}', liveUpdateSchema);  
  
      viewData = viewData.replace(tempStr, replaceStr);   

      fs.writeFile(viewPath, viewData, (err) => {
        if (err) {
          callback(err);
        } else {
          log.verbose(consts.LOG_PREFIX, 'Written file: ' + viewPath);
          callback();
        }
      });            
    }
  });
}

// Modify the ViewController file to include analytics code snippets
function modifyAnalyticsViewController(appDetails, callback) {
  var projectFolder = path.join(process.cwd(), appDetails.appName);
  var viewPath = path.join(projectFolder, appDetails.appName, 'ViewController.swift'); 
  log.verbose(consts.LOG_PREFIX, 'Reading file: ' + viewPath);
  fs.readFile(viewPath, 'utf-8', function (viewErr, viewData)  {
    if (viewErr) {
      callback(viewErr);
    } else { 
      let tempIndex = viewData.indexOf("}");
      let tempStr = viewData.substring(0, tempIndex - 1);     
      let replaceStr = tempStr.concat(consts.ANALYTICS_IOS_IMPL_CALL);    
      viewData = viewData.replace(tempStr, replaceStr);

      tempIndex = viewData.indexOf("}");
      tempStr = viewData.substring(0, tempIndex + 1);    
      replaceStr = tempStr.concat(consts.ANALYTICS_IOS_IMPL);  
      replaceStr = replaceStr.replace('{appDetails.appName}', appDetails.appName);
      viewData = viewData.replace(tempStr, replaceStr); 

      tempStr = viewData.substring(0, viewData.lastIndexOf("}") + 1);
      replaceStr = tempStr.concat(consts.OCLOGGER);  
      viewData = viewData.replace(tempStr, replaceStr);   

      fs.writeFile(viewPath, viewData, (err) => {
        if (err) {
          callback(err);
        } else {
          log.verbose(consts.LOG_PREFIX, 'Written file: ' + viewPath);
          callback();
        }
      });            
    }
  });
}

