/*
	Licensed Material - Property of IBM

	(C) Copyright 2015, 2016 IBM Corp.

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/


'use strict';

var async = require('async');
var Q = require('q');
var utils = require('./requestUtils');

const servicesMap = {
    push : getPush,
    liveupdate: getLiveUpdate,
}

module.exports.getServices = function(serverInfo, appName) {
  var deferred = Q.defer();

  var services = [];
  const keys = Object.keys(servicesMap);
  async.forEachSeries(keys, function (key, callback) {
    servicesMap[key](serverInfo, appName).then(function(service) {
      if (service && (Object.keys(service)).length > 0) {
        service.name = key;
        services.push(service);
      }
      callback();
    }).catch(function (error) {   
      callback(error);
    })    
  }, function(err) {
    if (err) {
      deferred.reject(err);
    } else {
      if (services.length > 0) {
        deferred.resolve(services);
      } else {
        deferred.resolve();
      }
    }
  });

  return deferred.promise;
}

function getPush(serverInfo, appName) {
  var deferred = Q.defer();

  var service = {};
  exportTags(`/mfpadmin/management-apis/2.0/runtimes/mfp/notifications/applications/${appName}/tags`, serverInfo, appName, []).then(function (tags) {
    if (tags && tags.length > 0) {
      service.tags = tags;
    }
    return getSettings(serverInfo, appName);
  }).then(function (settings) {
    if (settings && (Object.keys(settings)).length > 0 ) {
      service.settings = settings;
    }
    deferred.resolve(service);  
  }).catch(function (err) {
    deferred.resolve();
  })  
  
  return deferred.promise;                 
}

function exportTags(url, serverInfo, appName, tags) { 
  var deferred = Q.defer();

  getTags(url, serverInfo).then(function(response) {
    if (response && response.tags && response.tags.length > 0) {
      response.tags.forEach(function (responseTag) {
        var tag = {};
        tag.name = responseTag.name;
        tag.description = responseTag.description;
        tags.push(tag);
      });
    }     

    if (response && response.pageInfo && response.pageInfo.next) {
      var url = response.pageInfo.next;
      if (url.includes('?')) {
        var queryParams  = url.split('?')[1];
        url = `/mfpadmin/management-apis/2.0/runtimes/mfp/notifications/applications/${appName}/tags?` + queryParams;
      }
      exportTags(url, serverInfo, appName, tags).then(function(tags){
        deferred.resolve(tags);
      })
    } else {
        deferred.resolve(tags);
    }         
  }).catch(function (err) {
      deferred.resolve();       
  });  
  
  return deferred.promise;     
}

function getTags(url, serverInfo) {    
  var deferred = Q.defer();

  var options = {
    path : url,
    method : 'GET'
  }

  utils.sendRequest(serverInfo, options).then(function (response) {
    deferred.resolve(response);                  
  }).catch(function (err) {
    deferred.resolve();
  });
  
  return deferred.promise;
}

function getSettings(serverInfo, appName) {
  var deferred = Q.defer();

  var settings = {};
  getFcmConf(serverInfo, appName).then(function(result) {
    if (result) {
      settings.fcm = result;
    }
    return getApnsConf(serverInfo, appName);
  }).then(function(result) {
    if (result) {
      settings.apns = result;
    }
    deferred.resolve(settings);
  }).catch(function (err) {
    deferred.resolve();
  }); 
  
  return deferred.promise;
}

function getFcmConf(serverInfo, appName) {
  var deferred = Q.defer();

  var options = {
    path : `/mfpadmin/management-apis/2.0/runtimes/mfp/notifications/applications/${appName}/gcmConf`,
    method : 'GET'
  }

  utils.sendRequest(serverInfo, options).then(function (response) {
    if (response) {
      delete response.productVersion;
      deferred.resolve(response);
    } else {
      deferred.resolve();
    }
  }).catch(function (err) {
    deferred.resolve();
  });  
  
  return deferred.promise;
}

function getApnsConf(serverInfo, appName) {
  var deferred = Q.defer()

  var options = {
    path : `/mfpadmin/management-apis/2.0/runtimes/mfp/notifications/applications/${appName}/apnsConf`,
    method : 'GET'
  }

  utils.sendRequest(serverInfo, options).then(function (response) {
    if (response) {
      delete response.productVersion;      
      delete response.validUntil;
      response.password = '<<Insert password here>>';
      deferred.resolve(response);
    } else {
      deferred.resolve();
    }
  }).catch(function (err) {
    deferred.resolve();
  });  
  
  return deferred.promise;
}

function getLiveUpdate(serverInfo, appName) {
  var deferred = Q.defer();

  var service = {};
  getSchema(serverInfo, appName).then(function (schema) {
    if (schema && (Object.keys(schema)).length > 0) {
      if ((Object.keys(schema.features)).length > 0 || (Object.keys(schema.properties)).length  > 0) {
        service.schema = schema;
      }
    }
    deferred.resolve(service);
  }).catch(function (err) {
    deferred.resolve();
  }) 

  return deferred.promise;                   
}

function getSchema(serverInfo, appName) {
  var deferred = Q.defer()

  var options = {
    path : `/mfpadmin/management-apis/2.0/runtimes/mfp/mfpliveupdate/${appName}/schema`,
    method : 'GET'
  }
  utils.sendRequest(serverInfo, options).then(function (response) {
    if (response) {
      deferred.resolve(response);
    } else {
      deferred.resolve();
    }
  }).catch(function (err) {
    deferred.resolve();
  });  
  
  return deferred.promise;
}
