/*
	Licensed Material - Property of IBM

	(C) Copyright 2015, 2016 IBM Corp.

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/


'use strict';

var log = require('npmlog');
var Q = require('q');
var http = require('http');
var https = require('https');
var serverSupport = require('mdo-server-support');
var strings = require('ibm-strings');
var messages = require('./externalizedStrings');

var moduleName = 'requestUtils';

module.exports.sendRequest = function (serverInfo, options, payload) {
    var deferred = Q.defer();

    options.hostname = serverInfo.host;
    options.port = serverInfo.port;
    options.runtime = serverInfo.runtime;
    options.protocol = serverInfo.protocol + ':';

    options.headers = {
      'Content-Type' : 'application/json',
      'Authorization': 'Basic ' + new Buffer(serverInfo.login + ':' + serverInfo.password).toString('base64'),
      'Accept-Charset':'UTF-8',
      'User-Agent':''
    };
    var requestCallback = function(response) {
        var content = '';
        var errs;  

        response.on('data', function (data) {

          if (data) {
            try {
              content += decodeURI(data);
            } catch(err) {
              log.silly(moduleName, 'Failed to perform decodeURI: ' + err);
              content += data;
            }
            if (data.errors) errs = data.errors;
          }
        });
  
        response.on('end', function() {
          if (response.statusCode === 200 || response.statusCode === 201 ) {
            if (content !== '') {
              try {
                var results = JSON.parse(content);
                log.silly(moduleName, 'Results: ' + JSON.stringify(results, null, 2));
  
                if (results.errors && results.errors.length) {
                  strings.fatal(messages.logErrorLabel);
                  results.errors.forEach(function (error) {
                    strings.fatal(error);
                  });
                }
                if (results.warnings && results.warnings.length) {
                  strings.warn(messages.logWarningLabel);
                  results.warnings.forEach(function (warning) {
                    strings.warn(warning);
                  });
                }
              } catch (e) {
                strings.log(content);
              }
            } else {
              log.silly(moduleName, 'Received empty response');
            }
          } else {
            log.silly(moduleName, (strings.format(messages.statusCodeMessage,response.statusCode,http.STATUS_CODES[response.statusCode]))[response.statusCode > 400 ? strings.theme.error : strings.theme.warn]);
            if (content !== '') strings.log(JSON.parse(content));
          }
          deferred.resolve();
        });
    };
  
    var request, formattedParm;
    options.path = encodeURI(options.path);
 
    if (!formattedParm) {
        formattedParm = JSON.stringify(payload);
    }

    log.silly(moduleName, 'call push api:  \'%j\'', options);
    request = getRequest(options, requestCallback);

    if (options.method === 'POST' || options.method === 'PUT') {
        log.silly(moduleName, 'call push api: writing form data:\n', formattedParm);
        request.write(formattedParm);
    }
       

    request.on('error', function (e) {
        // If there's an error with the request, log it and resolve the deferral
        if (e.code === 'ECONNREFUSED') {
          strings.fatal(e.message);
          deferred.reject(e.code);
        }
        else {
          strings.log(messages.errorInvoking);
          strings.log(e.message, strings.theme.error);
        }
        deferred.reject();
    });
    request.end();

    return deferred.promise;
}

module.exports.sendFileRequest = function (serverInfo, options, payload) {
    var deferred = Q.defer();

    const FormData = require('form-data');
    const formData = new FormData();

    options.hostname = serverInfo.host;
    options.port = serverInfo.port;
    options.runtime = serverInfo.runtime;
    options.protocol = serverInfo.protocol + ':';

    options.headers = formData.getHeaders({'Authorization': 'Basic ' + new Buffer(serverInfo.login + ':' + serverInfo.password).toString('base64'), 'User-Agent':''});

    var requestCallback = function(response) {
        var content = '';
        var errs;
  
        response.on('data', function (data) {

          if (data) {
            try {
              content += decodeURI(data);
            } catch(err) {
              log.silly(moduleName, 'Failed to perform decodeURI: ' + err);
              content += data;
            }
            if (data.errors) errs = data.errors;
          }
        });
  
        response.on('end', function() {
          if (response.statusCode === 200 || response.statusCode === 201 ) {
            if (content !== '') {
              try {
                var results = JSON.parse(content);
                log.silly(moduleName, 'Results: ' + JSON.stringify(results, null, 2));
  
                if (results.errors && results.errors.length) {
                  strings.fatal(messages.logErrorLabel);
                  results.errors.forEach(function (error) {
                    strings.fatal(error);
                  });
                }
                if (results.warnings && results.warnings.length) {
                  strings.warn(messages.logWarningLabel);
                  results.warnings.forEach(function (warning) {
                    strings.warn(warning);
                  });
                }
              } catch (e) {
                strings.log(content);
              }
            } else {
              strings.warn(messages.emptyResponse);
            }
          } else {
            log.silly(moduleName, (strings.format(messages.statusCodeMessage,response.statusCode,http.STATUS_CODES[response.statusCode]))[response.statusCode > 400 ? strings.theme.error : strings.theme.warn]);
            if (content !== '') strings.log(JSON.parse(content));
          }
          deferred.resolve();
        });
    };
  
    var request, formattedParm;
    options.path = encodeURI(options.path);
 
    if (!formattedParm) {
        formattedParm = JSON.stringify(payload);
    }

    log.silly(moduleName, 'call push api:  \'%j\'', options);
    request = getRequest(options, requestCallback);

    Object.keys(payload).forEach(function(keyName) {
        formData.append(keyName, payload[keyName]);
    })
    formData.pipe(request);

    // if (options.method === 'POST' || options.method === 'PUT') {
    //     log.silly(moduleName, 'call push api: writing form data:\n', formattedParm);
    //     request.write(formattedParm);
    // }
       

    request.on('error', function (e) {
        // If there's an error with the request, log it and resolve the deferral
        if (e.code === 'ECONNREFUSED') {
          strings.fatal(e.message);
          deferred.reject(e.code);
        }
        else {
          strings.log(messages.errorInvoking);
          strings.log(e.message, strings.theme.error);
        }
        deferred.reject();
    });
    //request.end();

    return deferred.promise;
}

function getRequest(options, callback) {
    serverSupport.getCertificate(options, moduleName);
  
    if (options.protocol === 'https:') {
      log.silly(moduleName, 'performing https request');
      return https.request(options, callback);
    } else {
      log.silly(moduleName, 'performing http request');
      return http.request(options, callback);
    }
  }
