/*
	Licensed Material - Property of IBM

	(C) Copyright 2015, 2016 IBM Corp.

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/


'use strict';

var moduleName = 'services';

var async = require('async');
var fs = require('fs');
var findup = require('findup-sync');
var path = require('path');
var Q = require('q');
var utils = require('./requestUtils');

const servicesMap = {
    push : configurePush,
    liveupdate: configureLiveUpdate,
}

module.exports.configureServices = function(appJson, serverInfo, appName) {
    var deferred = Q.defer();

    if (appJson.services) {
        async.forEachSeries(appJson.services, function (service, callback) {
            var serviceName = service.name;
            if (Object.keys(servicesMap).includes(serviceName)) {
                servicesMap[serviceName](service, serverInfo, appName, function (err) {
                    if (err) {
                        callback(err);
                    } else {
                        callback();
                    }
                });
            } else {
                callback();
            }
        }, function (err) {
            if (err) {
                deferred.reject(err);
            } else {
                deferred.resolve();
            }
        });
    } else {
        deferred.resolve();
    }   

    return deferred.promise;
}

function configurePush(pushConfig, serverInfo, appName, callback) {
  async.parallel([
      postTags.bind(null, pushConfig, serverInfo, appName),
      putFcmConf.bind(null, pushConfig, serverInfo, appName),
      putApnsConf.bind(null, pushConfig, serverInfo, appName),
  ], function(err, results) {
      if (err) {
          callback(err);
      } else {
          callback();
      }
  });                                                   
}

function postTags(pushConfig, serverInfo, appName, callback) {    
    if (pushConfig.tags && pushConfig.tags.length > 0) {
      async.forEachSeries(pushConfig.tags, function(tag, innerCallback) {
          var options = {
            path : `/mfpadmin/management-apis/2.0/runtimes/mfp/notifications/applications/${appName}/tags`,
            method : 'POST'
          }
          utils.sendRequest(serverInfo, options, tag).then(function (response) {
              innerCallback();                       
          }).catch(function (err) {
              innerCallback(err);
          });              
      }, function(err) {
          if (err) {
              callback(err);
          } else {
              callback();
          }
      });
    } else {
        callback();
    }
}

function putFcmConf(pushConfig, serverInfo, appName, callback) {
    if (pushConfig.settings && pushConfig.settings.fcm) {
      var options = {
        path : `/mfpadmin/management-apis/2.0/runtimes/mfp/notifications/applications/${appName}/gcmConf`,
        method : 'PUT'
      }      
      utils.sendRequest(serverInfo, options, pushConfig.settings.fcm).then(function (response) {
          callback();                       
      }).catch(function (err) {
          callback(err);
      });              
    } else {
        callback();
    }
}

function putApnsConf(pushConfig, serverInfo, appName, callback) {
    if (pushConfig.settings && pushConfig.settings.apns) {
      var options = {
        path : `/mfpadmin/management-apis/2.0/runtimes/mfp/notifications/applications/${appName}/apnsConf`,
        method : 'PUT'
      }      
      const mfpConfigPath = findup('mfpconfig');
      const payload = {
          'certificate' : fs.createReadStream(path.join(mfpConfigPath, pushConfig.settings.apns.certificate)),
          'isSandBox' : pushConfig.settings.apns.isSandBox.toString(),
          'password' : pushConfig.settings.apns.password
      }      
      utils.sendFileRequest(serverInfo, options, payload).then(function (response) {
          callback();                       
      }).catch(function (err) {
          callback(err);
      });              
    } else {
        callback();
    }
}

function configureLiveUpdate(liveUpdateConfig, serverInfo, appName, callback) { 
  async.parallel([
      putSchema.bind(null, liveUpdateConfig, serverInfo, appName),
  ], function(err, results) {
      if (err) {
          callback(err);
      } else {
          callback();
      }
  });                                                   
}

function putSchema(liveUpdateConfig, serverInfo, appName, callback) {
  if (liveUpdateConfig.schema) {
      var options = {
        path : `/mfpadmin/management-apis/2.0/runtimes/mfp/mfpliveupdate/${appName}/schema`,
        method : 'PUT'
      }
      utils.sendRequest(serverInfo, options, liveUpdateConfig.schema).then(function (response) {
          callback();                       
      }).catch(function (err) {
          callback(err);
      });              
  } else {
      callback();
  }
}
